/*
 * A user-configured filesystem server.
 *
 * Copyright (C) 2018, 2019, 2020 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <ext2fs/ext2fs.h>
#include <systypes/user.h>

/* Implemented interface. */

#include <fsserver/user_filesystem_resource.h>



/* Filesystem server for a specific user identity. */

class Fs_user_filesystem : public UserFilesystemResource
{
private:
  ext2_filsys _fs;
  const char *_devname;
  user_t _user;

protected:
  /* Helper methods. */

  long _check_access(int flags, ext2_ino_t ino_file);

  long _create(const char *path, ext2_ino_t ino_parent, ext2_ino_t *ino_file);

  long _find(const char *path, int flags, int *ext2flags, ext2_ino_t *ino_file);

  /* Obtain an accessor for a file. */

  Accessor *_get_accessor(ext2_ino_t ino_file, int flags=0);

  /* Obtain a resource for a file. */

  Resource *_get_resource(ext2_ino_t ino_file, fs_object_t *fsobj, Accessor *accessor);

public:
  explicit Fs_user_filesystem(Paging *paging, ext2_filsys fs,
                              const char *devname, user_t user)
  : UserFilesystemResource(paging), _fs(fs), _devname(devname), _user(user)
  {
  }

  /* A mechanism for obtaining servers for filesystem objects.
     Here, the default is overridden with an e2fs-specific implementation. */

  virtual long open_object_resource(fs_object_t *fsobj, int flags,
                                    Resource **resource);
};
